
import type { EdgeWorker } from '../classes.edgeworker.js';
import type { WorkerEvent } from '../classes.workerevent.js';
import * as plugins from '../plugins.js';
import { SmartlogDestination } from './smartlog.js';

export interface IAnalyticsData {
  requestAgent: string;
  requestUrl: string;
  requestMethod: string;
  requestStartTime: number;
  responseStatus: number;
  responseEndTime: number;
}

export class Analyzer {
  cworkerEventRef: WorkerEvent;

  public data: IAnalyticsData = {
    requestAgent: 'unknown',
    requestMethod: 'unknown',
    requestUrl: 'unknown',
    requestStartTime: 0,
    responseStatus: 0,
    responseEndTime: 0,
  };

  public finishedDeferred = plugins.smartpromise.defer();

  constructor(cworkerEventRefArg: WorkerEvent) {
    this.cworkerEventRef = cworkerEventRefArg;
    this.smartlog.addLogDestination(new SmartlogDestination(this.cworkerEventRef.options.edgeWorkerRef));
  }
  public smartlog = new plugins.smartlog.Smartlog({
    logContext: {
      environment: 'production',
      runtime: "cloudflare_workers",
      zone: 'servezone',
      company: 'Lossless GmbH',
      companyunit: 'Lossless Cloud',
      containerName: 'cloudflare_workers'
    }
  });

  public setRequestData (optionsArg: {
    requestAgent: string;
    requestUrl: string;
    requestMethod: string;
  }) {
    this.data = {
      ...this.data,
      ...{
        requestAgent: optionsArg.requestAgent,
        requestUrl: optionsArg.requestUrl,
        requestMethod: optionsArg.requestMethod,
        requestStartTime: Date.now()
      }
    };
    
  }

  public setResponseData(optionsArg: {
    responseStatus: number,
    responseEndTime: number,
  }) {
    this.data = {
      ...this.data,
      ...{
        responseStatus: optionsArg.responseStatus,
        responseEndTime: optionsArg.responseEndTime
      }
    };
    this.sendLogs();
  }

  public async sendLogs() {
    await this.smartlog.log('info', `
      Got a ${this.data.requestMethod} request from ${this.data.requestAgent} to
      ${this.data.requestUrl}
      that took ${this.data.responseEndTime - this.data.requestStartTime}ms to resolve with status ${this.data.responseStatus}.`, this.data);
    this.finishedDeferred.resolve();
  }
}